# Copyright (c) SenseTime. All Rights Reserved.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import os
import sys
env_path = os.path.join(os.path.dirname(__file__), '..')
print(env_path)
if env_path not in sys.path:
    sys.path.append(env_path)
import argparse
import os

import cv2
import torch
import numpy as np

from pysot_toolkit.bbox import get_axis_aligned_bbox
from pysot_toolkit.toolkit.datasets import DatasetFactory
from pysot_toolkit.toolkit.utils.region import vot_overlap, vot_float2str
from pysot_toolkit.trackers.tracker import Tracker
from pysot_toolkit.trackers.tracker_nc import Tracker as Tracker_nc
from pysot_toolkit.trackers.tracker_ar_online import TrackingNC_ALPHA_ON as Tracker_nc_on
from pysot_toolkit.trackers.tracker_ar_motion import TrackingNC_motion as Tracker_nc_motion
from pysot_toolkit.trackers.tracker_ar_sz import TrackingNC_ALPHA_sz as Tracker_nc_sz
from pysot_toolkit.trackers.tracker_ar_segm_test import TrackingNC_Segm_t as TrackingNC_Segm_t
from pysot_toolkit.trackers.tracker_ar_1 import TrackingNC_ALPHA
from pysot_toolkit.trackers.tracker_ar_box import TrackingNC_ALPHA_box
from pysot_toolkit.trackers.tracker_ar_sz_st_hp import TrackingNC_ALPHA_st

#from pysot_toolkit.trackers.tracker_ar_sz_hp import TrackingNC_ALPHA_sz
from pysot_toolkit.trackers.net_wrappers import NetWithBackbone
from pysot_toolkit.trackers.vot20_utils import *
import random

parser = argparse.ArgumentParser(description='siamrpn tracking')
parser.add_argument('--dataset', type=str,default='VOT2019', #GOT-10k 'VOT2018
        help='datasets')
parser.add_argument('--video', default='', type=str,
        help='eval one special video')
parser.add_argument('--vis', default=False,
        help='whether visualzie result')
parser.add_argument('--name', default='transtcross_ft', type=str,
        help='name of results')
args = parser.parse_args()

torch.set_num_threads(1)



def main():
    def set_seed(seed):
        random.seed(seed)
        os.environ['PYTHONHASHSEED'] = str(seed)
        np.random.seed(seed)
        torch.manual_seed(seed)
        torch.cuda.manual_seed(seed)
        torch.backends.cudnn.benchmark = True
        torch.backends.cudnn.deterministic = True
        torch.backends.cudnn.enabled = True
    seed_test = 1000
    set_seed(seed_test)

    # load config

    dataset_root = '/home/v-xfei/Public/dataset/vot_tools/vot19/sequences/' #Absolute path of the dataset
    #dataset_root = '/home/v-xfei/Public/dataset/vottrain/got10k/test/' #Absolute path of the dataset

   # net_path = '/home/v-xfei/Public/project/TransT/pytracking/networks/CrossViT_22k_ep0013.pth.tar'  # 65
    #net_path = '/home/v-xfei/Public/project/TransT/pytracking/networks/CrossViT_22k_ep0400.pth.tar'
    net_path = '/home/v-xfei/Public/project/TransT/pytracking/networks/CrossViT_ep0099.pth.tar'

    # create model
    print(net_path)
    net = NetWithBackbone(net_path=net_path, use_gpu=True)
    #from ltr.models.tracking.transcross import transcrossSeg_vit
    #net = transcrossSeg_vit()
    #weight = torch.load(net_path)['net']
    #new_weight = dict()
    #for k, v in weight.items():
     #   new_weight[k[7:]] = v
    #net.load_state_dict(new_weight)
   # tracker = TrackingNC_Segm_t(name='transcross', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224, search_z=3, search_x=3) #window_penalty=0.49
   # tracker = TrackingNC_ALPHA_box(name='transt_ar', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224)
    #tracker = TrackingNC_ALPHA(name='transt_ar', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224)
    tracker = TrackingNC_ALPHA_st(net = net, refine_path='/home/v-xfei/Public/project/TransT/pysot_toolkit/ar.pth.tar')
   # tracker = Tracker(name='transt', net=net, window_penalty=0.49, exemplar_size=128, instance_size=256)
    #tracker = Tracker_nc_on(name='transtcross_ft', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224, search_z=3, search_x=3) #window_penalty=0.49
    #tracker = Tracker_nc_motion(name='transtcross_ft', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224, search_z=3, search_x=3) #window_penalty=0.49
    #tracker =Tracker_nc_sz(net=net, refine_path='/home/v-xfei/Public/project/TransT/pysot_toolkit/ar.pth.tar')
    # create dataset
    dataset = DatasetFactory.create_dataset(name=args.dataset,
                                            dataset_root=dataset_root,
                                            load_img=False)

    model_name = args.name
    total_lost = 0
    if args.dataset in ['VOT2016', 'VOT2018', 'VOT2019']:
        # restart tracking
        for v_idx, video in enumerate(dataset):
            #args.video = ['ball3', 'book', 'conduction1','dribble','hand2','rabbit','crabs1','scoccer1']
           # args.video = [ 'basketball']

            if args.video != '':
                # test one special video
                if video.name not in args.video:
                    continue
            frame_counter = 0
            lost_number = 0
            toc = 0
            pred_bboxes = []
            for idx, (img, gt_bbox) in enumerate(video):
                if True: #video.name in ['ball3']:
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                    if len(gt_bbox) == 4:
                        gt_bbox = [gt_bbox[0], gt_bbox[1],
                                   gt_bbox[0], gt_bbox[1] + gt_bbox[3] - 1,
                                   gt_bbox[0] + gt_bbox[2] - 1, gt_bbox[1] + gt_bbox[3] - 1,
                                   gt_bbox[0] + gt_bbox[2] - 1, gt_bbox[1]]
                    tic = cv2.getTickCount()
                    if idx == frame_counter:
                        cx, cy, w, h = get_axis_aligned_bbox(np.array(gt_bbox))
                        gt_bbox_ = [cx - w / 2, cy - h / 2, w, h]
                        init_info = {'init_bbox': gt_bbox_}
                        tracker.initialize(img, init_info)
                        pred_bbox = gt_bbox_
                        pred_bboxes.append(1)
                    elif idx > frame_counter:
                        info = {}
                        outputs = tracker.track(img, info)
                        if isinstance(outputs, dict):
                           pred_bbox = outputs['target_bbox']
                        else:
                           pred_bbox = outputs[0]

                        # if cfg.MASK.MASK:
                        #     pred_bbox = outputs['polygon']
                        overlap = vot_overlap(pred_bbox, gt_bbox, (img.shape[1], img.shape[0]))
                        if overlap > 0:
                            # not lost
                            pred_bboxes.append(pred_bbox)
                        else:
                            # lost object
                            pred_bboxes.append(2)
                            frame_counter = idx + 5#5  # skip 5 frames
                            lost_number += 1
                            #print('lost')
                    else:
                        pred_bboxes.append(0)
                    toc += cv2.getTickCount() - tic
                    if idx == 0:
                        cv2.destroyAllWindows()
                    if args.vis and idx > frame_counter:
                        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
                        cv2.polylines(img, [np.array(gt_bbox, np.int).reshape((-1, 1, 2))],
                                      True, (0, 255, 0), 3)
                        bbox = list(map(int, pred_bbox))
                        cv2.rectangle(img, (bbox[0], bbox[1]),
                                      (bbox[0] + bbox[2], bbox[1] + bbox[3]), (0, 255, 255), 3)
                        cv2.putText(img, str(idx), (40, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 255), 2)
                        cv2.putText(img, str(lost_number), (40, 80), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 255), 2)
                        cv2.imshow(video.name, img)
                        if cv2.waitKey() & 0xFF == ord('q'):
                            break
                else:
                    pass

            toc /= cv2.getTickFrequency()
            # save results
            video_path = os.path.join('results', args.dataset, model_name,
                    'baseline', video.name)
            if not os.path.isdir(video_path):
                os.makedirs(video_path)
            result_path = os.path.join(video_path, '{}_001.txt'.format(video.name))
            with open(result_path, 'w') as f:
                for x in pred_bboxes:
                    if isinstance(x, int):
                        f.write("{:d}\n".format(x))
                    else:
                        f.write(','.join([vot_float2str("%.4f", i) for i in x])+'\n')
            print('({:3d}) Video: {:12s} Time: {:4.1f}s Speed: {:3.1f}fps Lost: {:d}'.format(
                    v_idx+1, video.name, toc, idx / toc, lost_number))
            total_lost += lost_number
        print("{:s} total lost: {:d}".format(model_name, total_lost))
    else:
        # OPE tracking
        for v_idx, video in enumerate(dataset):
            if args.video != '':
                # test one special video
                if video.name != args.video:
                    continue
            toc = 0
            pred_bboxes = []
            scores = []
            track_times = []
            for idx, (img, gt_bbox) in enumerate(video):
                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                tic = cv2.getTickCount()
                if idx == 0:
                    cx, cy, w, h = get_axis_aligned_bbox(np.array(gt_bbox))
                    gt_bbox_ = [cx-w/2, cy-h/2, w, h]
                    init_info = {'init_bbox':gt_bbox_}
                    tracker.initialize(img, init_info)
                    #tracker.initialize(img, gt_bbox_)

                    pred_bbox = gt_bbox_
                    scores.append(None)
                    if 'VOT2018-LT' == args.dataset:
                        pred_bboxes.append([1])
                    else:
                        pred_bboxes.append(pred_bbox)
                else:
                    outputs = tracker.track(img)
                    if isinstance(outputs, dict):
                        pred_bbox = outputs['target_bbox']
                        bscore = outputs['best_score']

                    else:
                        pred_bbox = outputs[0]
                        bscore =  outputs[1]
                    #outputs = tracker.track(img)
                    #pred_bbox = outputs  # outputs['target_bbox']
                    pred_bboxes.append(pred_bbox)
                    scores.append(bscore)

                toc += cv2.getTickCount() - tic
                track_times.append((cv2.getTickCount() - tic)/cv2.getTickFrequency())
                if idx == 0:
                    cv2.destroyAllWindows()
                if args.vis and idx > 0:
                    img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
                    gt_bbox = list(map(int, gt_bbox))
                    pred_bbox = list(map(int, pred_bbox))
                    cv2.rectangle(img, (gt_bbox[0], gt_bbox[1]),
                                  (gt_bbox[0]+gt_bbox[2], gt_bbox[1]+gt_bbox[3]), (0, 255, 0), 3)
                    cv2.rectangle(img, (pred_bbox[0], pred_bbox[1]),
                                  (pred_bbox[0]+pred_bbox[2], pred_bbox[1]+pred_bbox[3]), (0, 255, 255), 3)
                    cv2.putText(img, str(idx), (40, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 255), 2)
                    cv2.imshow(video.name, img)
                    cv2.waitKey(1)
            toc /= cv2.getTickFrequency()
            # save results
            if 'VOT2018-LT' == args.dataset:
                video_path = os.path.join('results', args.dataset, model_name,
                        'longterm', video.name)
                if not os.path.isdir(video_path):
                    os.makedirs(video_path)
                result_path = os.path.join(video_path,
                        '{}_001.txt'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in pred_bboxes:
                        f.write(','.join([str(i) for i in x])+'\n')
                result_path = os.path.join(video_path,
                        '{}_001_confidence.value'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in scores:
                        f.write('\n') if x is None else f.write("{:.6f}\n".format(x))
                result_path = os.path.join(video_path,
                        '{}_time.txt'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in track_times:
                        f.write("{:.6f}\n".format(x))
            elif 'GOT-10k' == args.dataset:
                video_path = os.path.join('results', args.dataset, model_name, video.name)
                if not os.path.isdir(video_path):
                    os.makedirs(video_path)
                result_path = os.path.join(video_path, '{}_001.txt'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in pred_bboxes:
                        f.write(','.join([str(i) for i in x])+'\n')
                result_path = os.path.join(video_path,
                        '{}_time.txt'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in track_times:
                        f.write("{:.6f}\n".format(x))
            else:
                model_path = os.path.join('results', args.dataset, model_name)
                if not os.path.isdir(model_path):
                    os.makedirs(model_path)
                result_path = os.path.join(model_path, '{}.txt'.format(video.name))
                with open(result_path, 'w') as f:
                    for x in pred_bboxes:
                        f.write(','.join([str(i) for i in x])+'\n')
            print('({:3d}) Video: {:12s} Time: {:5.1f}s Speed: {:3.1f}fps'.format(
                v_idx+1, video.name, toc, idx / toc))


if __name__ == '__main__':
    main()
